@@ -305,6 +305,14 @@ class CLIParser:
                 help="the Object Store directory name to use when uploading objects",
                 metavar="<directory>",
             )
+            parser.add_argument(
+                "--content-path",
+                "-C",
+                type=str,
+                required=False,
+                help="the path where the items to be uploaded are found",
+                metavar="<directory>",
+            )
             parser.add_argument(
                 "filenames",
                 metavar="<filename>",
@@ -529,6 +537,10 @@ class CLIParser:
     def task_batch_size(self) -> Optional[int]:
         return self.args.task_batch_size
 
+    @property
+    def content_path(self) -> Optional[str]:
+        return self.args.content_path
+
 
 def lookup_module_description(module_name: str) -> Optional[str]:
     """
