@@ -23,6 +23,7 @@ from yd_commands.args import ARGS_PARSER
 from yd_commands.config_keys import *
 from yd_commands.mustache import (
     UTCNOW,
+    add_substitutions,
     load_toml_file_with_mustache_substitutions,
     substitute_mustache_str,
 )
@@ -47,6 +48,9 @@ YD_NAMESPACE = "YD_NAMESPACE"
 YD_TAG = "YD_TAG"
 YD_URL = "YD_URL"
 
+TASK_BATCH_SIZE_DEFAULT = 2000
+DEFAULT_URL = "https://portal.yellowdog.co/api"
+
 
 @dataclass
 class ConfigWorkRequirement:
@@ -71,22 +75,27 @@ class ConfigWorkRequirement:
     max_retries: int = 0
     max_workers: Optional[int] = None
     min_workers: Optional[int] = None
+    optional_inputs: List[Dict] = field(default_factory=list)
     output_files: List[str] = field(default_factory=list)
+    output_files_required: List[str] = field(default_factory=list)
     priority: float = 0.0
     providers: Optional[List[str]] = None
     ram: Optional[List[float]] = None
     regions: Optional[List[str]] = None
+    task_batch_size: int = TASK_BATCH_SIZE_DEFAULT
     task_count: int = 1
+    task_data: Optional[str] = None
+    task_data_file: Optional[str] = None
     task_name: Optional[str] = None
     task_group_name: Optional[str] = None
     task_type: Optional[str] = None
-    tasks_data_file: Optional[str] = None
     tasks_per_worker: Optional[int] = None
     upload_files: List[Dict] = field(default_factory=list)
     vcpus: Optional[List[float]] = None
     verify_at_start: List[str] = field(default_factory=list)
     verify_wait: List[str] = field(default_factory=list)
     worker_tags: Optional[List[str]] = None
+    wr_data_file: Optional[str] = None
     wr_name: Optional[str] = None
 
 
@@ -187,17 +196,19 @@ def load_config_common() -> ConfigCommon:
                     f"for '{key_name}'"
                 )
 
-        DEFAULT_URL = "https://portal.yellowdog.co/api"
         url = substitute_mustache_str(common_section.get(URL, DEFAULT_URL))
         if url != DEFAULT_URL:
             print_log(f"Using the YellowDog API at: {url}")
+        namespace = substitute_mustache_str(common_section[NAMESPACE])
+        name_tag = substitute_mustache_str(common_section[NAME_TAG])
+        add_substitutions(subs={NAMESPACE: namespace, NAME_TAG: name_tag, URL: url})
 
         return ConfigCommon(
             # Required
             key=substitute_mustache_str(common_section[KEY]),
             secret=substitute_mustache_str(common_section[SECRET]),
-            namespace=substitute_mustache_str(common_section[NAMESPACE]),
-            name_tag=substitute_mustache_str(common_section[NAME_TAG]),
+            namespace=namespace,
+            name_tag=name_tag,
             # Optional
             url=url,
         )
@@ -238,11 +249,11 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
             for index, worker_tag in enumerate(worker_tags):
                 worker_tags[index] = substitute_mustache_str(worker_tag)
 
-        tasks_data_file = wr_section.get(WR_DATA, None)
-        if tasks_data_file is not None:
-            check_str(tasks_data_file)
-            tasks_data_file = substitute_mustache_str(tasks_data_file)
-            tasks_data_file = pathname_relative_to_config_file(tasks_data_file)
+        wr_data_file = wr_section.get(WR_DATA, None)
+        if wr_data_file is not None:
+            check_str(wr_data_file)
+            wr_data_file = substitute_mustache_str(wr_data_file)
+            wr_data_file = pathname_relative_to_config_file(wr_data_file)
 
         # Check for properties set on the command line
         executable = (
@@ -269,6 +280,12 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
         if csv_file:
             csv_files = [csv_file]
 
+        task_batch_size = (
+            wr_section.get(TASK_BATCH_SIZE, TASK_BATCH_SIZE_DEFAULT)
+            if ARGS_PARSER.task_batch_size is None
+            else ARGS_PARSER.task_batch_size
+        )
+
         return ConfigWorkRequirement(
             args=wr_section.get(ARGS, []),
             bash_script=wr_section.get(BASH_SCRIPT, None),  # Deprecated
@@ -293,22 +310,27 @@ def load_config_work_requirement() -> Optional[ConfigWorkRequirement]:
             max_retries=wr_section.get(MAX_RETRIES, 0),
             max_workers=wr_section.get(MAX_WORKERS, None),
             min_workers=wr_section.get(MIN_WORKERS, None),
+            optional_inputs=wr_section.get(OPTIONAL_INPUTS, []),
             output_files=wr_section.get(OUTPUT_FILES, []),
+            output_files_required=wr_section.get(OUTPUT_FILES_REQUIRED, []),
             priority=wr_section.get(PRIORITY, 0.0),
             providers=wr_section.get(PROVIDERS, None),
             ram=wr_section.get(RAM, None),
             regions=wr_section.get(REGIONS, None),
+            task_batch_size=task_batch_size,
             task_count=wr_section.get(TASK_COUNT, 1),
+            task_data=wr_section.get(TASK_DATA, None),
+            task_data_file=wr_section.get(TASK_DATA_FILE, None),
             task_group_name=wr_section.get(TASK_GROUP_NAME, None),
             task_name=wr_section.get(TASK_NAME, None),
             task_type=task_type,
-            tasks_data_file=tasks_data_file,
             tasks_per_worker=wr_section.get(TASKS_PER_WORKER, None),
             upload_files=wr_section.get(UPLOAD_FILES, []),
             vcpus=wr_section.get(VCPUS, None),
             verify_at_start=wr_section.get(VERIFY_AT_START, []),
             verify_wait=wr_section.get(VERIFY_WAIT, []),
             worker_tags=worker_tags,
+            wr_data_file=wr_data_file,
             wr_name=wr_section.get(WR_NAME, None),
         )
     except KeyError as e:
