@@ -47,14 +47,16 @@ from yd_commands.csv_data import (
 )
 from yd_commands.interactive import confirmed
 from yd_commands.mustache import (
-    LAZY_SUBS_WRAPPER,
-    TASK_COUNT_SUB,
-    TASK_GROUP_COUNT_SUB,
-    TASK_GROUP_NUMBER_SUB,
-    TASK_NUMBER_SUB,
+    TASK_COUNT,
+    TASK_GROUP_COUNT,
+    TASK_GROUP_NUMBER,
+    TASK_NUMBER,
+    WR_NAME,
+    add_substitutions,
     load_json_file_with_mustache_substitutions,
     load_jsonnet_file_with_mustache_substitutions,
     load_toml_file_with_mustache_substitutions,
+    process_mustache_substitutions,
 )
 from yd_commands.printing import (
     WorkRequirementSnapshot,
@@ -63,7 +65,11 @@ from yd_commands.printing import (
     print_log,
     print_numbered_strings,
 )
-from yd_commands.submit_utils import UploadedFiles, generate_task_input_list
+from yd_commands.submit_utils import (
+    UploadedFiles,
+    format_yd_name,
+    generate_task_input_list,
+)
 from yd_commands.type_check import (
     check_bool,
     check_dict,
@@ -92,7 +98,6 @@ UPLOADED_FILES: Optional[UploadedFiles] = None
 
 @main_wrapper
 def main():
-
     if ARGS_PARSER.json_raw:
         submit_json_raw(ARGS_PARSER.json_raw)
         return
@@ -202,7 +207,12 @@ def submit_work_requirement(
 
     # Overwrite the WR name?
     global ID
-    ID = wr_data.get(WR_NAME, ID if CONFIG_WR.wr_name is None else CONFIG_WR.wr_name)
+    ID = format_yd_name(
+        wr_data.get(WR_NAME, ID if CONFIG_WR.wr_name is None else CONFIG_WR.wr_name)
+    )
+    # Lazy substitution of the Work Requirement name, now it's defined
+    add_substitutions(subs={WR_NAME: ID})
+    process_mustache_substitutions(wr_data)
 
     # Handle any files that need to be uploaded
     global UPLOADED_FILES
@@ -292,11 +302,13 @@ def create_task_group(
     # Name the Task Group
     num_task_groups = len(wr_data[TASK_GROUPS])
     num_tasks = len(task_group_data[TASKS])
-    task_group_name = get_task_group_name(
-        task_group_data.get(NAME, CONFIG_WR.task_group_name),
-        tg_number,
-        num_task_groups,
-        num_tasks,
+    task_group_name = format_yd_name(
+        get_task_group_name(
+            task_group_data.get(NAME, CONFIG_WR.task_group_name),
+            tg_number,
+            num_task_groups,
+            num_tasks,
+        )
     )
 
     # Assemble the RunSpecification values for the Task Group;
@@ -461,7 +473,6 @@ def add_tasks_to_task_group(
 
     # Iterate through batches
     for batch_number in range(num_task_batches):
-
         # Iterate through tasks in the batch
         for task_number in range(
             (TASK_BATCH_SIZE * batch_number),
@@ -469,12 +480,14 @@ def add_tasks_to_task_group(
         ):
             task_group_data = wr_data[TASK_GROUPS][tg_number]
             task = tasks[task_number] if task_count is None else tasks[0]
-            task_name = get_task_name(
-                task.get(NAME, CONFIG_WR.task_name),
-                task_number,
-                num_tasks,
-                tg_number,
-                num_task_groups,
+            task_name = format_yd_name(
+                get_task_name(
+                    task.get(NAME, CONFIG_WR.task_name),
+                    task_number,
+                    num_tasks,
+                    tg_number,
+                    num_task_groups,
+                )
             )
             executable = check_str(
                 task.get(
@@ -521,9 +534,21 @@ def add_tasks_to_task_group(
                     ),
                 )
             )
+            optional_inputs_list = check_list(
+                task.get(
+                    OPTIONAL_INPUTS,
+                    task_group_data.get(
+                        OPTIONAL_INPUTS,
+                        wr_data.get(OPTIONAL_INPUTS, CONFIG_WR.optional_inputs),
+                    ),
+                )
+            )
 
             check_for_duplicates_in_file_lists(
-                input_files_list, verify_at_start_files_list, verify_wait_files_list
+                input_files_list,
+                verify_at_start_files_list,
+                verify_wait_files_list,
+                optional_inputs_list,
             )
 
             # Upload files in the 'inputs' list
@@ -569,10 +594,13 @@ def add_tasks_to_task_group(
             inputs += generate_task_input_list(
                 verify_wait_files_list, TaskInputVerification.VERIFY_WAIT, ID
             )
+            inputs += generate_task_input_list(
+                files=optional_inputs_list, verification=None, wr_name=ID
+            )
 
             # Set up the 'outputs' property
             outputs = [
-                TaskOutput.from_worker_directory(file)
+                TaskOutput.from_worker_directory(file_pattern=file, required=False)
                 for file in check_list(
                     task.get(
                         OUTPUT_FILES,
@@ -584,6 +612,26 @@ def add_tasks_to_task_group(
                 )
             ]
 
+            # Add the contents of the 'outputsRequired' property
+            outputs += [
+                TaskOutput.from_worker_directory(file_pattern=file, required=True)
+                for file in check_list(
+                    task.get(
+                        OUTPUT_FILES_REQUIRED,
+                        task_group_data.get(
+                            OUTPUT_FILES_REQUIRED,
+                            wr_data.get(
+                                OUTPUT_FILES_REQUIRED, CONFIG_WR.output_files_required
+                            ),
+                        ),
+                    )
+                )
+            ]
+
+            # Set 'alwaysUpload' to true for all outputs
+            for task_output in outputs:
+                task_output.alwaysUpload = True
+
             # Add TaskOutput to 'outputs'?
             if check_bool(
                 task.get(
@@ -728,33 +776,19 @@ def cleanup_on_failure(work_requirement: WorkRequirement) -> None:
     UPLOADED_FILES.delete()
 
 
-def check_for_duplicates_in_file_lists(
-    input_files_list: List[str],
-    verify_at_start_list: List[str],
-    verify_wait_list: List[str],
-):
+def check_for_duplicates_in_file_lists(*args: List[str]):
     """
     Tests for duplicates in file lists. If duplicates found, print an error
     and raise an Exception.
     """
-    input_files_set = set(input_files_list)
-    verify_at_start_set = set(verify_at_start_list)
-    verify_wait_set = set(verify_wait_list)
-    intersection = input_files_set.intersection(verify_at_start_set)
-    if len(intersection) != 0:
-        raise Exception(
-            f"Duplicate files in 'inputs' and 'verifyAtStart' lists: {intersection}"
-        )
-    intersection = input_files_set.intersection(verify_wait_set)
-    if len(intersection) != 0:
-        raise Exception(
-            f"Duplicate files in 'inputs' and 'verifyWait' lists: {intersection}"
-        )
-    intersection = verify_at_start_set.intersection(verify_wait_set)
-    if len(intersection) != 0:
-        raise Exception(
-            f"Duplicate files in 'verifyWait' and 'verifyAtStart' lists: {intersection}"
-        )
+    files_list = []
+    for file_list in args:
+        files_list += file_list
+    files_list_unique = list(set(files_list))
+    for file in files_list_unique:
+        files_list.remove(file)
+    if len(files_list) != 0:
+        raise Exception(f"Duplicate file(s) in file lists: {files_list}")
 
 
 def formatted_number_str(current_item_number: int, num_items: int) -> str:
@@ -779,19 +813,19 @@ def get_task_name(
 
     if name:
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_NUMBER_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_NUMBER}}}}}",
             formatted_number_str(task_number, num_tasks),
         )
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_COUNT}}}}}",
             str(num_tasks),
         )
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_NUMBER_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_GROUP_NUMBER}}}}}",
             formatted_number_str(task_group_number, num_task_groups),
         )
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_GROUP_COUNT}}}}}",
             str(num_task_groups),
         )
 
@@ -814,15 +848,15 @@ def get_task_group_name(
 
     if name:
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_NUMBER_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_GROUP_NUMBER}}}}}",
             formatted_number_str(task_group_number, num_task_groups),
         )
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_GROUP_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_GROUP_COUNT}}}}}",
             str(num_task_groups),
         )
         name = name.replace(
-            f"{LAZY_SUBS_WRAPPER}{TASK_COUNT_SUB}{LAZY_SUBS_WRAPPER}",
+            f"{{{{{TASK_COUNT}}}}}",
             str(task_count),
         )
 
@@ -926,7 +960,6 @@ def create_task(
     # password if specified.
     elif task_type == "docker":
         if executable is None:
-            print_log("Note: no 'executable' specified for 'docker' Task Type")
             return _make_task(flatten_input_paths)
 
         # Set up the environment variables to be sent to the Docker container
@@ -998,8 +1031,14 @@ def submit_json_raw(wr_file: str):
             f"Work Requirement file '{wr_file}' must end in '.json' or '.jsonnet'"
         )
 
+    # Lazy substitution of Work Requirement name
+    wr_data["name"] = format_yd_name(wr_data["name"])
+    wr_name = wr_data["name"]
+    add_substitutions(subs={WR_NAME: wr_name})
+    process_mustache_substitutions(wr_data)
+
     if ARGS_PARSER.dry_run:
-        # This will show the results any Mustache processing
+        # This will show the results of any Mustache processing
         print_log("Dry-run: Printing JSON Work Requirement specification:")
         print_json(wr_data)
         print_log("Dry-run: Complete")
@@ -1017,14 +1056,13 @@ def submit_json_raw(wr_file: str):
         task_lists[task_group["name"]] = task_group.get("tasks", [])
         task_group.pop("tasks", None)
 
-    # Submit the Work Requirement
+    # Submit the Work Requirement and its Task Groups
     response = requests.post(
         url=f"{CONFIG_COMMON.url}/work/requirements",
         headers={"Authorization": f"yd-key {CONFIG_COMMON.key}:{CONFIG_COMMON.secret}"},
         json=wr_data,
     )
 
-    wr_name = wr_data["name"]
     if response.status_code == 200:
         wr_id = jsons.loads(response.text)["id"]
         print_log(f"Created Work Requirement '{wr_name}' [{wr_id}]")
@@ -1033,13 +1071,13 @@ def submit_json_raw(wr_file: str):
         raise Exception(f"{response.text}")
 
     # Submit Tasks to the Work Requirement
+    # Collect 'VERIFY_AT_START' files
     verify_at_start_files = set()
     for task_group_name, task_list in task_lists.items():
-
         # Collect set of VERIFY_AT_START files
         for task in task_list:
             for input in task.get("inputs", []):
-                if input["verification"] == "VERIFY_AT_START":
+                if input.get("verification", None) == "VERIFY_AT_START":
                     namespace = (
                         CONFIG_COMMON.namespace
                         if input["source"] == "TASK_NAMESPACE"
@@ -1049,24 +1087,25 @@ def submit_json_raw(wr_file: str):
                         f"{namespace} :: {input['objectNamePattern']}"
                     )
 
-        # Warn about VERIFY_AT_START files & halt to allow upload or
-        # Work Requirement cancellation
-        if len(verify_at_start_files) != 0:
-            print_log(
-                "The following files may be required ('VERIFY_AT_START') "
-                "before Tasks are submitted, or the Tasks will fail."
-            )
-            print_log(
-                "You now have an opportunity to upload the required files "
-                "before Tasks are submitted:\n"
-            )
-            print_numbered_strings(sorted(list(verify_at_start_files)))
-            if not confirmed("Proceed now (y), or Cancel Work Requirement (n)?"):
-                print_log(f"Cancelling Work Requirement '{wr_name}'")
-                CLIENT.work_client.cancel_work_requirement_by_id(wr_id)
-                return
+    # Warn about VERIFY_AT_START files & halt to allow upload or
+    # Work Requirement cancellation
+    if len(verify_at_start_files) != 0:
+        print_log(
+            "The following files may be required ('VERIFY_AT_START') "
+            "before Tasks are submitted, or the Tasks will fail."
+        )
+        print_log(
+            "You now have an opportunity to upload the required files "
+            "before Tasks are submitted:\n"
+        )
+        print_numbered_strings(sorted(list(verify_at_start_files)))
+        if not confirmed("Proceed now (y), or Cancel Work Requirement (n)?"):
+            print_log(f"Cancelling Work Requirement '{wr_name}'")
+            CLIENT.work_client.cancel_work_requirement_by_id(wr_id)
+            return
 
-        # Submit Tasks in batches
+    # Submit Tasks in batches
+    for task_group_name, task_list in task_lists.items():
         num_batches = ceil(len(task_list) / TASK_BATCH_SIZE)
         for index in range(num_batches):
             task_batch = task_list[
